package edu.uoc.zonefeed;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

public class OntologyParser {
	OntModel model;
	
	public OntologyParser() {
		
	}
	
	public void loadOntology(String url) {
	    OntModel model = ModelFactory.createOntologyModel();

	    model.read(url);
	    
	    this.model = model;
	}
	
	private void addSubCategories(OntModel model, OntClass parentClass, CategoryNode classNode) {
		Iterator<OntClass> sublist = parentClass.listSubClasses(true);
		
		while (sublist.hasNext()) {
			OntClass current = sublist.next();
			String currentName = getName(current);
			if (currentName != null) {
				CategoryNode newChild = new CategoryNode(currentName);
				String newDescription = current.getComment("EN");
				if (newDescription != null) {
					newChild.setDescription(newDescription);
				}
				addSubCategories(model, current, newChild);
				classNode.addChild(newChild);
			}
			
		}
	}
	
	private void addSubCategories(OntModel model, CategoryNode target) {
		Iterator<OntClass> list = model.listClasses();

		while (list.hasNext()) {
			OntClass current = list.next();
			String currentName = getName(current);
			if ((currentName != null) && (currentName.equals(target.getValue()))) {
				// Populate properties
				addSubCategories(model, current, target);
				break;
			}
		}
	}
	
	public CategoryNode getCategoryTree(String baseName) {
		CategoryNode root = new CategoryNode(baseName);
		
		addSubCategories(model, root);
		
		return root;
	}
	
	public List<CategoryNode> getAllClasses() {
		List<CategoryNode> result = new ArrayList<CategoryNode>();
		
		Iterator<OntClass> list = model.listClasses();
		while (list.hasNext()) {
			OntClass next = list.next();
			String name = getName(next);
			if (name != null) {
				CategoryNode newChild = new CategoryNode(name);
				result.add(newChild);
			}
		}
		return result;
	}
	
	private String getName(OntResource target) {
		String result = null;
		
		if (target == null) return null;
		
		if (target.getLabel("EN") != null) {
			result = target.getLabel("EN");
		} else if (target.getLocalName() != null) {
			result = target.getLocalName();
		}
		
		return result;
	}
}
