package edu.uoc.zonefeed;

import com.googlecode.objectify.*;
import com.google.appengine.api.users.User;
import java.util.List;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.Iterator;

public class Persistance {
  private static Persistance instance = null;
  private Objectify ofy;

  public static Persistance getInstance() {
	  if (instance == null) {
		  instance = new Persistance();
	  }
	  return instance;
  }
  
  private Persistance() {
	  ObjectifyService.register(Categories.class);
	  ObjectifyService.register(Comment.class);
	  ObjectifyService.register(CommentScore.class);
	  ObjectifyService.register(Story.class);
	  ObjectifyService.register(RegisteredUser.class);
	  ObjectifyService.register(StoredImage.class);
	  ObjectifyService.register(Property.class);
	  ObjectifyService.register(CategoryProperty.class);
	  
	  ofy = ObjectifyService.begin();
  }
  
  public long storeStory(Story target) {
	  Key<Story> newKey = ofy.put(target);
	  
	  return newKey.getId();
  }
  
  public void storeComment(Comment target) {
	  ofy.put(target);
  }
  
  public Key<StoredImage> storeImage(StoredImage target) {
	  return ofy.put(target);
  }
  
  public Key<RegisteredUser> storeUser(RegisteredUser target) {
	  return ofy.put(target);
  }
  
  public Key<CategoryProperty> storeCategoryProperty(CategoryProperty target) {
	  return ofy.put(target);
  }
  
  public Key<Property> storeProperty(Property target) {
	  return ofy.put(target);
  }
  
  public Categories getCategories() {
	  return ofy.query(Categories.class).get();
  }
  
  public String[] getAllCategoryProperties() {
	 List<CategoryProperty> allProperties = ofy.query(CategoryProperty.class).list();
	 Iterator<CategoryProperty> entries = allProperties.iterator();
	 LinkedHashSet<String> returnValues = new LinkedHashSet<String>();
	 
	 while (entries.hasNext()) {
		 CategoryProperty entry = entries.next();
		 returnValues.add(entry.getName());
	 }
	 
	 return returnValues.toArray(new String[0]);
  }
  
  public Story[] searchProperties(Map<String,String> param) {
	  LinkedHashSet<Key<Story>> stories = new LinkedHashSet<Key<Story>>();
	  Set<String> keySet = param.keySet();
	  
	  Iterator<String> keys = keySet.iterator();
	  boolean isFirstIteration = true;
	  
	  while (keys.hasNext()) {
		  String currentKey = keys.next();
		  LinkedHashSet<Key<Story>> currentStories = new LinkedHashSet<Key<Story>>();
		  Query<Property> propertyQuery = ofy.query(Property.class).filter("name", currentKey).filter("value", param.get(currentKey));
		  List<Property> propertyList = propertyQuery.list();
		  
		  
		  Iterator<Property> properties =  propertyList.iterator();
		  
		  while (properties.hasNext()) {
			  Property currentProperty = properties.next();
			  if (isFirstIteration || stories.contains(currentProperty.getStory())) {
				  currentStories.add(currentProperty.getStory());
			  }
		  }
		  stories = currentStories;
		  isFirstIteration = false;
	  }
	  
	  
	  return ofy.get(stories).values().toArray(new Story[0]);
  }
  
  public List<CategoryProperty> getCategoryProperties(String category) {
	  return ofy.query(CategoryProperty.class).filter("category", category).list();
  }
  
  public CategoryProperty getCategoryProperty(long target) {
	  return ofy.get(new Key<CategoryProperty>(CategoryProperty.class, target));
  }
  
  public void removeCategoryProperty(CategoryProperty target) {
	  ofy.delete(target);
  }
  
  public void storeCategories(Categories categories) {
	  ofy.put(categories);
  }
  
  public List<Property> getStoryProperties(long id) {
	  return ofy.query(Property.class).filter("story", new Key<Story>(Story.class, id)).list();
  }
  
  public Story getStory(long id) {
	  return ofy.get(Story.class, id);
  }
  
  public Comment[] getStoryComments(long id) {
	  Query<Comment> q = ofy.query(Comment.class).order("-created").filter("story", new Key<Story>(Story.class, id));
	  List<Comment> l = q.list();
	  return l.toArray(new Comment[0]);
  }
  
  public RegisteredUser getRegisteredUser(String email) {
	  try {
		  return ofy.get(new Key<RegisteredUser>(RegisteredUser.class, email));
	  } catch (NotFoundException nfex) {
		  return null;
	  }
  }
  
  public StoredImage getImage(long id) {
	  return ofy.get(new Key<StoredImage>(StoredImage.class, id));
  }
  
  public RegisteredUser getRegisteredUserByScreenName(String screenName) {
	  return ofy.query(RegisteredUser.class).filter("screenName", screenName).get();
  }
  
  public Story[] getUserStories(User target) {
	  Query<Story> q = ofy.query(Story.class).order("-created").filter("author", target);
	  List<Story> l = q.list();
	  return l.toArray(new Story[0]);
  }
  
  public Story[] getCategoryStories(String category) {
	  Query<Story> q = ofy.query(Story.class).order("-created").filter("category", category);
	  List<Story> l = q.list();
	  return l.toArray(new Story[0]);
  }
  
  public Story[] getActiveStories() {
	  Query<Story> q = ofy.query(Story.class).order("-modified");
	  List<Story> l = q.list();
	  return l.toArray(new Story[0]);
  }
  
  public Comment getComment(long id) {
	  return ofy.get(new Key<Comment>(Comment.class, id));
  }
  
  public CommentScore getCommentScore(long comment, User voter) {
	  return ofy.query(CommentScore.class).filter("comment", new Key<Comment>(Comment.class, comment)).filter("author", voter).get();
  }
  
  public Key<CommentScore> storeCommentScore(CommentScore target) {
	  return ofy.put(target);
  }
}
